#!/usr/bin/env python

#   bibbob: a simple sound generator
#   Copyright (C) 2010  Niels Serup

#   This file is part of bibbob.
#
#   bibbob is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   bibbob is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with bibbob.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>

##''''''''''' Documentation and downloads are available at '''''''''''##
################# http://metanohi.org/projects/bibbob/ #################

#### Parser

import sys
import re

from music import *
from sound import *
import drawwave

class Parser:
    def __init__(self, **settings):
        try:
            self.text = settings['text']
        except Exception:
            try:
                self.text = settings['file'].read()
            except Exception:
                self.err('No text specified.', True)
        self.text = self.text.replace(
            '\r\n', '\n').replace('\r','\n')

        self.channels = None
        self.rate = None
        self.std_music = None

        self.wave_classes = {
            'sine': SineWave,
            'square': SquareWave,
            'triangle': TriangleWave,
            'sawtooth': SawtoothWave,
            'sawtoothRev': SawtoothWaveReversed,
            'silence': SilenceSound,
            'random': RandomSound,
            'doubleSine': DoubleSine,
            'piano': PianoSynth,
            'flute': FluteSynth,
            'violin': ViolinSynth
        }
        self.sounds = {}
        self.tracks = []
        self.music = None

    def err(self, msg, fatal=False):
        sys.stderr.write('Error: %s' % msg)
        if fatal:
            sys.exit(1)

    def create_std_music(self):
        if self.channels is None:
            self.channels = 1
        if self.rate is None:
            self.rate = 44100

        self.std_music = Music(channels=self.channels, rate=self.rate)

    def remove_comments(self):
        self.text = re.sub(r'#.*', '', self.text)

    def remove_whitespace(self):
        self.text = re.sub(r'[ \t]+', ' ', self.text)

    def parse(self):
        self.remove_comments()
        self.remove_whitespace()
        line_num = 1
        for x in self.text.split('\n'):
            x = x.strip()
            if not x:
                continue
            args = x.split(' ')
#            self.parse_arguments(args)
            try:
                self.parse_arguments(args)
            except Exception, error:
                self.err('Line %d could not be parsed:\n%s' %
                         (line_num, error))
            line_num += 1

    def parse_arguments(self, args):
        key = args[0]
        if   key == 'channels' and self.channels is None:
            self.channels = int(args[1])
        elif key == 'rate' and self.rate is None:
            self.rate = int(args[1])
        elif key == 'wave':
            wave = args[1]

            if self.std_music is None:
                self.create_std_music()

            freq = 500 # Default
            amp = 1000 # values
            for arg in args[2:]:
                first = arg[:1]
                last = arg[1:]
                if   first == 'f':
                    freq = int(last)
                elif first == 'a':
                    amp = int(last)
                elif first == 'l':
                    length = int(last)
                elif first == '=':
                    name = last

            self.sounds[name] = self.std_music.create(
                self.wave_classes[wave], length, amp, freq)
        elif key == 'group' and self.sounds:
            name = args[-1]
            s = self.std_music.create(Sound, 0)
            for x in args[1:-1]:
                s.put(self.sounds[x])
            if s.length > 0:
                if name.startswith('='):
                    name = name[1:]
                self.sounds[name] = s
        elif key == 'track' and self.sounds:
            track = Music(channels=self.channels, rate=self.rate)
            for x in args[1:]:
                track.add(self.sounds[x])
            if track:
                self.tracks.append(track)
        elif key == 'create' and self.tracks:
            m = Music(channels=self.channels, rate=self.rate)
            m.put(self.tracks[0])
            m.combine(*self.tracks[1:])
            self.music = m
        elif key == 'play' and self.music:
            self.music.play()
        elif key == 'draw' and self.music:
            drawwave.make_png(self.music, width=1280, height=640)
