#!/usr/bin/env python

#   bibbob: a simple sound generator
#   Copyright (C) 2010  Niels Serup

#   This file is part of bibbob.
#
#   bibbob is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   bibbob is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with bibbob.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>

##''''''''''' Documentation and downloads are available at '''''''''''##
################# http://metanohi.org/projects/bibbob/ #################

#### Sounds


import numpy
import math
import random

import synthsounds as synth

class Sound:
    def __init__(self, channels, rate, length):
        self.channels = channels
        self.rate = rate
        self.length = length
        self.size = (channels * rate * length) / 1000
        self.data = numpy.empty(self.size, dtype=numpy.int16)

    def put(self, sound):
        self.data = numpy.append(self.data, sound.data)
        self.length += sound.length

    def combine(self, sound):
        if self.size > sound.size:
            l = sound.size
            nl = self.size
            s = self.data
        else:
            l = self.size
            nl = sound.size
            s = sound.data

        d1 = self.data
        d2 = sound.data
        for i in xrange(l):
            s[i] = (d1[i] + d2[i]) / 2

        self.data = s
        self.length = nl

    def create(self, *args):
        pass

class SilenceSound(Sound):
    def create(self, *args):
        for x in xrange(self.size):
            self.data[x] = 0

class RandomSound(Sound):
    def create(self, amplitude, *args):
        chanrange = range(self.channels)
        for x in xrange(0, self.size, self.channels):
            t = random.randint(-amplitude, amplitude)
            for j in chanrange:
                self.data[x + j] = t

class SineWave(Sound):
    def create(self, amplitude, frequency):
        chanrange = range(self.channels)
        i = 0
        angular_frequency = 2 / ((self.rate / float(frequency)) / math.pi)
        for x in xrange(0, self.size, self.channels):
            t = int(amplitude * math.sin(angular_frequency * i))
            for j in chanrange:
                self.data[x + j] = t
            i += 1

class SquareWave(Sound):
    def create(self, amplitude, frequency):
        width = self.rate / frequency / 2
        height = amplitude
        chanrange = range(self.channels)
        num = height
        i = 0
        for x in xrange(0, self.size, self.channels):
            for j in chanrange:
                self.data[x + j] = num
            i += 1
            if i == width:
                num = -num
                i = 0

class TriangleWave(Sound):
    def create(self, amplitude, frequency):
        width = self.rate / frequency / 4
        height = amplitude
        chanrange = range(self.channels)
        a = float(height) / width
        i = -width
        s = 1
        for x in xrange(0, self.size, self.channels):
            t = int(a * i)
            for j in chanrange:
                self.data[x + j] = t

            if s == 1 and i == width:
                s = -1
            elif s == -1 and i == -width:
                s = 1
            i += s

class SawtoothWave(Sound):
    def create(self, amplitude, frequency, rise=True):
        width = self.rate / frequency / 2
        height = amplitude
        chanrange = range(self.channels)
        a = float(height) / width
        if rise:
            i = -width
        else:
            i = width
        for x in xrange(0, self.size, self.channels):
            t = int(a * i)
            for j in chanrange:
                self.data[x + j] = t

            if rise:
                if i == width:
                    i = -width
                i += 1
            else:
                if i == -width:
                    i = width
                i -= 1

class SawtoothWaveReversed(SawtoothWave):
    def create(self, amplitude, frequency):
        SawtoothWave.create(self, amplitude, frequency, False)

class DoubleSine(Sound):
    def create(self, amplitude, *args):
        chanrange = range(self.channels)
        i = 0
        part_angular_frequency = 2 / (self.rate / math.pi)
        current_frequency = 0
        for x in xrange(0, self.size, self.channels):
            if i % 10 == 0:
                current_frequency = 200 * math.sin(i*0.0002)
            #print current_frequency
            t = int(amplitude * math.sin(part_angular_frequency *
                                         current_frequency * i))
            for j in chanrange:
                self.data[x + j] = t
            i += 1

def synthwave(f, var):
    for i in range(0, len(var), 3):
        if f > var[i]:
            return var[i + 1] * (f - var[i]) + var[i + 2]

class Synth(Sound):
    def __init__(self, *args):
        Sound.__init__(self, *args)
        self.set_list()

    def set_list(self):
        pass

    def use_list(self, lst):
        self.list = list(lst[:])
        self.listtype = self.list[:]
        for i in range(len(self.listtype)):
            self.listtype[i] = 1

    def change_list(self):
        for i in range(3, len(self.list), 3):
#            x1 = self.list[i]
 #           y1 = self.list[i+2]
  #          x0 = self.list[i-3]
   #         y0 = self.list[i-1]
    #        a = self.list[i-2]

#            self.list[i+2] = math.fmod((self.list[i+2] + (self.list[i]
#                                        ** 2) ** 0.1 + 1.0), 2.0) - 1.0
            self.list[i+2] += self.list[i] / 10.0 * self.listtype[i]
            if self.list[i+2] > 1.0:
                self.list[i+2] = 1.0
                self.listtype[i] *= -1
            elif self.list[i+2] < -1.0:
                self.list[i+2] = -1.0
                self.listtype[i] *= -1

            try:
                self.list[i-2] = (self.list[i+2] - self.list[i-1]) / \
                    (self.list[i] - self.list[i-3])
            except ZeroDivisionError:
                pass



    def create(self, amplitude, frequency):
        chanrange = range(self.channels)
        i = 0
        nf = self.rate / float(frequency)
        for x in xrange(0, self.size, self.channels):
            pos = (i % nf) / nf
            if i % (nf * 10) == 0:
                self.change_list()
            t = int(amplitude * synthwave(pos, self.list))
            for j in chanrange:
                self.data[x + j] = t
            i += 1

class PianoSynth(Synth):
    def set_list(self):
        self.use_list(synth.piano)

class FluteSynth(Synth):
    def set_list(self):
        self.use_list(synth.flute)

class ViolinSynth(Synth):
    def set_list(self):
        self.use_list(synth.violin)
