#!/usr/bin/env python

#   mustdo: a todo list program with unnecessary regular expressions
#   Copyright (C) 2010  Niels Serup

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>
#   v0.1

import sys
import os.path
import re

class Opts:
	def __init__(self, args):
		self.filename = args[0]
		self.mode = self.get_mode(args[1])
		self.message = ' '.join(args[2:])
	
	def get_mode(self, long):
		letter = long[:1]
		if long == '' or long == '--help' or long == '-h':
			return '?'
		elif long == '--version' or long == '-v':
			return 'v'
		elif letter == '0':
			return '0'
		elif letter == '-':
			return '-'
		else:
			return '+'

class MustDoList:
	def __init__(self, opts):
		self.filename = opts.filename
		self.mode = opts.mode
		self.message = opts.message
		
		if not os.path.exists(self.filename):
			self.create_file()
		self.open_file()
		
		self.has_changed = False
		self.has_outputted = False
		
		if self.mode == '+':
			self.append_message()
		elif self.mode == '-':
			self.remove_messages()
			self.remove_nones()
		elif self.mode == '?':
			self.display_help()
		elif self.mode == 'v':
			self.display_version()
		
		self.close_file()
		
		if not self.has_outputted:
			length = len(self.data)
			lenstr = len(str(length - 1))
			for i in range(length):
				i_mod = str(i)
				i_mod = '0' * (lenstr - len(i_mod)) + i_mod
				print "%s. %s" % (i_mod, self.data[i])
		
		if self.has_changed:
			self.save_to_file()
				
	
	def create_file(self):
		open(self.filename, 'w').close()
	
	def open_file(self):
		self.file = open(self.filename, 'U')
		self.data = self.file.read().strip('\r\n').split('\n')
		if self.data[0] == '':
			del self.data[0]
		self.file.close()
	
	def close_file(self):
		self.file.close()
	
	def save_to_file(self):
		self.file = open(self.filename, 'w')
		self.file.write('\n'.join(self.data))
	
	def append_message(self):
		if not self.message:
			return
		self.data.append(self.message)
		self.has_changed = True
	
	def remove_messages(self):
		if not self.message:
			return
		
		self.has_changed = True
		
		if re.match(r"^[0-9]+$", self.message):
			try:
				self.data[int(self.message)] = None
			except Exception:
				pass
		elif re.match(r"^[0-9]+\s*-\s*[0-9]+$", self.message):
			ints = [int(x) for x in self.message.split('-')]
			for y in range(ints[0], ints[1] + 1):
				try:
					self.data[y] = None 
				except Exception:
					pass
		elif re.match(r"^([0-9]+\s*,\s*)+[0-9]+$", self.message):
			ints = [int(x) for x in self.message.split(',')]
			for x in ints:
				try:
					self.data[x] = None
				except Exception:
					pass
		elif re.match(r"^([0-9]+\s*-\s*[0-9]+\s*,\s*)+[0-9]+\s*-\s*[0-9]+$", self.message):
			ints = [[int(y) for y in x.split('-')] for x in self.message.split(',')]
			for x in ints:
				for y in range(x[0], x[1] + 1):
					try:
						self.data[y] = None 
					except Exception:
						pass
		elif re.match(r"^(([0-9]+|[0-9]+\s*-\s*[0-9]+)\s*,\s*)+([0-9]+|[0-9]+\s*-\s*[0-9]+)$", self.message):
			parts = self.message.split(',')
			orig_message = self.message[:]
			for x in parts:
				self.message = x
				self.remove_messages()
			self.message = orig_message
		else:
			self.has_changed = False
	
	def remove_nones(self):
		while None in self.data:
			self.data.remove(None)
	
	def display_help(self):
		#pn = sys.argv[0]
		pn = "mustdo"
		print """\
Usage: %s [MODE|-(h|v|-help|-version)] [MESSAGE]
Adds or removes messages from a must do list

Examples:
  Adding a message to the list:
    %s + Bake a cake

  Listing the messages:
    %s 0
   Or just:
    %s

  Removing one message:
    %s - <n>
  
  Removing a series of messages:
    %s - <n1>-<n2>
  
  Removing a list of messages:
    %s - <n1>,<n2>-<n3>,<n4>-<n5>,<n6>

Spaces are allowed in MESSAGE.\
""" % (pn, pn, pn, pn, pn, pn, pn)
		self.has_outputted = True
	
	def display_version(self):
		print """\
mustdo 0.1
Copyright (C) 2010  Niels Serup <ns@metanohi.org>
License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.\
"""
		self.has_outputted = True


if __name__ == "__main__":
	filename = "%s/.mustdo-list" % os.path.expanduser('~')
	args = sys.argv[1:]
	args.insert(0, filename)
	
	while len(args) < 3:
		args.append('')
	
	opts = Opts(args)
	must = MustDoList(opts)
