#!/usr/bin/env python

#  xptv2smil: a PiTiVi XPTV file to a SMIL file converter
#  Copyright (C) 2009  Niels Serup

#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

### Maintained by Niels Serup <ns@metanohi.org> ###


def convert_xptv_to_smil(xptv_data, et):
	# XPTV data is pure XML
	# SMIL data is pure XML
	
	# Definitions
	smil_data_start = """\
<?xml version="1.0"?>
<smil xmlns="http://www.w3.org/2001/SMIL20/Language">
	<body>\
"""
	
	smil_data_entry = """
		<seq>
			<video src="%s" clipBegin="%s" clipEnd="%s"/>
		</seq>\
"""
	
	smil_data_end = """\
	</body>
</smil>\
"""
	
	# Parsing
	smil_data = smil_data_start[:]
	
	root = et.XML(xptv_data)	
	sources = root.find('factories/sources')
	track_objects = root.find('timeline/tracks/track/track-objects')
	
	#####################
	def get_filename(id):
		filename = None
		
		for node in sources:
			if node.get('id') == id:
				filename = node.get('filename')
				if filename.find('file://') == 0:
					filename = filename[7:]
				break
		
		return filename
	#####################
	def zero_in_front(tstr):
		return '0' * (2 - len(tstr)) + tstr
	#####################
	def get_seconds(tstr):
		if tstr.find('(gint64)') == 0:
			tstr = tstr[8:]
			seconds = int(tstr) / 1000000000.0
			return seconds
		else:
			return 0
	#####################
	def time_format(tstr, skip_p1 = False):
		if (isinstance(tstr, basestring) and tstr.find('(gint64)') == 0) or skip_p1:
			if not skip_p1:
				tstr = tstr[8:]
				time_seconds = int(tstr) / 1000000000.0
				seconds = int(time_seconds)
				seconds_strd = str(time_seconds)
			else:
				seconds_strd = str(tstr)
				seconds = int(tstr)
			
			milliseconds = seconds_strd[seconds_strd.find('.')+1:][:3]
			milliseconds += '0' * (3 - len(milliseconds))
			
			hours = seconds / 3600
			seconds -= hours * 3600
			
			minutes = seconds / 60
			seconds -= minutes * 60
			
			seconds = zero_in_front(str(seconds))
			minutes = zero_in_front(str(minutes))
			hours = zero_in_front(str(hours))
			
			return '%s:%s:%s.%s' % (hours, minutes, seconds, milliseconds)
		else:
			return '00:00:00.000'
	#####################
	
	for node in track_objects:
		id = node.find('factory-ref').get('id')
		filename = get_filename(id)
		#timeline_start = time_format(node.get('start'))
		start = node.get('in_point')
		
		start_secs = get_seconds(start)
		duration_secs = get_seconds(node.get('duration'))
		end_secs = start_secs + duration_secs
		
		start = time_format(start)
		end = time_format(end_secs, True)
		
		smil_data += smil_data_entry % (filename, start, end)
	
	smil_data += '\n' + smil_data_end
	
	return smil_data

def load():
	import sys

	def err(msg, exit = 1):
		sys.stderr.write(msg+'\n')
		if exit:
			sys.exit(exit)

	try:
		import xml.etree.cElementTree as et
	except:
		try:
			import xml.etree.ElementTree as et
		except:
			try:
				import lxml.etree as et
			except:
				err('No ElementTree present.')

	args = sys.argv[1:]
	
	filename = None
	data     = None
	
	if len(args) > 0:
		if args[0] == '-h' or args[0] == '--help':
			print """\
Usage: xptv2smil FILE|-(h|-help|v|-version)
Converts a PiTiVi XPTV file to a SMIL file

If no FILE is specified, xptv2smil will attempt to grab xptv data
from standard in.

xptv2smil will print the SMIL output to standard out.

xptv2smil isn't the type of program that writes a lot of error messages.\
"""
			sys.exit(0)
		elif args[0] == '-v' or args[0] == '--version':
			print """\
xptv2smil 0.1
Copyright (C) 2009  Niels Serup
License LGPLv3+: GNU LGPL version 3 or later <http://gnu.org/licenses/lgpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.\
"""
			sys.exit(0)
		else:
			filename = args[0]
	
	if not filename:
		if not sys.stdin.isatty():
			try:
				data = sys.stdin.read()
			except:
				pass
	else:
		try:
			f = open(filename, 'U')
			data = f.read()
			f.close()
		except:
			pass
		
	if not data:
		err('No data. Enter "xptv2smil --help" for more info.')
	
	# If data seems to be present, go on:
	print convert_xptv_to_smil(data, et)

if __name__ == '__main__':
	load()
